# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# forked from: http://pypi.python.org/pypi/hgrecipe/0.9
# Qua 16 Mar 2011 09:09:01 BRT
import logging
import os
import shutil

from mercurial import commands, hg, ui

class Mercurial(object):
	"""
	Buildout Recipe to clone from a Mercurial Repository.

	``directory``
		The file path where the repository will be cloned to. Defaults
		to a folder matching the name of the repository in the Buildout
		parts directory.

	``repository``
		The url of the repository to clone.

	``newest``
		Whether to pull updates from the source repository during
		subsequent Buildouts. This can be specified on the recipe or
		at the Buildout level.

	``overwrite``
		Whether to clobber an existing repository on install and have
		Buildout remove on uninstallation or to keep it around always.
	"""

	def __init__(self, buildout, name, options):
		options['directory'] = options.get('directory') or \
			os.path.join(buildout['buildout']['parts-directory'], name)

		self.logger = logging.getLogger(name)

		self.repository = options.get('repository')
		self.directory = options['directory']
		self.newest = options.get('newest',
			buildout['buildout'].get('newest', 'true')).lower() != 'false'

		self.overwrite = options.get('overwrite') == 'true'

	def install(self):
		"""
		Install the repository specified by the ``repository`` argument
		of the recipe.

		If it doesn't exist, clone it to the specified ``directorys``,
		and if it does exist try pulling into it.

		If ``overwrite`` is marked as true in the recipe, an existing
		repositories in ``directory`` will be clobbered.

		If ``overwrite`` isn't true, don't return any directories for
		Buildout to remove on uninstall.
		"""
		self.logger.info("Cloning repository %s to %s" % (
			self.repository, self.directory
		))

		if self.overwrite:
			shutil.rmtree(self.directory, ignore_errors=True)

		if not os.path.exists(self.directory):
			self.clone()
		else:
			self.pull()

		if self.overwrite:
			return self.directory
		return ""

	def clone(self):
		hg.clone(ui.ui(), self.repository, self.directory)

	def pull(self):
		commands.pull(ui.ui(), hg.repository(ui.ui(), self.directory),
			self.repository, update=True)

	def update(self):
		"""
		Pull updates from the upstream repository.

		If ``newest`` is set to False in the recipe or in the buildout
		configuration, no action is taken.
		"""
		if self.newest:
			self.logger.info("Pulling repository %s and updating %s" % (
				self.repository, self.directory
			))
			commands.pull(ui.ui(), hg.repository(ui.ui(), self.directory),
				self.repository, update=True)

def uninstall(name, options):
	"""
	Remove the old repository if ``overwrite`` is marked as true.
	Otherwise, leave it alone.
	"""
	if options.get('overwrite') == 'true':
		shutil.rmtree(options.get('directory'), ignore_errors=True)



# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Ter 15 Mar 2011 18:03:35 BRT 

import hashlib
import os

TRUE_VALUES = ('yes', 'true', '1', 'on')

def get_bool_option(option):
    return option.strip().lower() in TRUE_VALUES

def get_checksum(path, hashtype='sha1'):
    if not os.path.isfile(path):
        return None

    func = getattr(hashlib, hashtype)
    checksum = func()

    f = open(path, 'rb')
    try:
        chunk = f.read(2**16)
        while chunk:
            checksum.update(chunk)
            chunk = f.read(2**16)

        return checksum.hexdigest()
    finally:
        f.close()



# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# borrowed from: http://pypi.python.org/pypi/iwm.recipe.svncheckout/0.0.1
# Ter 15 Mar 2011 14:01:03 BRT 
import logging, os
import pkg_resources

import zc.buildout
import zc.recipe.egg

from subprocess import Popen, PIPE

def cmd(cmdline):
	p = Popen(cmdline.split(), stdout=PIPE)
	return p.communicate()[0]

class Subversion:
	def __init__(self, buildout, name, options):
		self.name, self.options = name, options
		self.logger = logging.getLogger(self.name)
		self.root_directory = buildout['buildout']['directory']

		self.target = os.path.join(self.root_directory, options['directory'])

		options.setdefault('revision', 'HEAD')
		self.revision = options.get('revision')

		self.url = options['url']
		self.newest = (
			buildout['buildout'].get('offline', 'false') == 'false' and \
			buildout['buildout'].get('newest', 'true') == 'true'
		)

	def update(self):
		"""
		Does nothing if buildout is in offline mode.
		"""
		if not self.newest:
			return self.target

		os.chdir(self.target)
		update_command = 'svn up -r %s' % self.revision
		self.logger.info(update_command)
		cmd(update_command)

		return self.target

	def install(self):
		"""
		Checkout a working copy. Fails if buildout is
		running in offline mode.
		"""
		os.chdir(self.root_directory)
		install_command = 'svn co -r %s %s %s' % (self.revision, self.url, self.target)
		self.logger.info(install_command)
		cmd(install_command)
		return self.target

def uninstall(name, options):
	"""
	Remove the old repository if ``overwrite`` is marked as true.
	Otherwise, leave it alone.
	"""
	if options.get('overwrite') == 'true':
		import shutil
		shutil.rmtree(options.get('directory'), ignore_errors=True)



# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Qua 16 Mar 2011 11:21:13 BRT 
"""
Downloads libraries from PyPi and installs in the app directory. This recipe
extends `zc.recipe.egg.Scripts <http://pypi.python.org/pypi/zc.recipe.egg>`_,
so all the options from that recipe are also valid.

Options
~~~~~~~

:eggs: Package names to be installed.
:lib-directory: Destination directory for the libraries. Default is
	`distlib`.
:use-zipimport: If `true`, a zip file with the libraries is created
	instead of a directory. The zip filename will be the value of
	`lib-directory` plus `.zip`.
:ignore-globs: A list of glob patterns to not be copied from the library.
:ignore-packages: A list of top-level package names or modules to be ignored.
	This is useful to ignore dependencies that won't be used. Some packages may
	install distribute, setuptools or pkg_resources but these are not very
	useful on App Engine, so you can set them to be ignored, for example.
:delete-safe: If `true`, always move `lib-directory` to a temporary directory
	inside the parts dir as a backup when building, instead of deleting it.
	This is to avoid accidental deletion if `lib-directory` is badly
	configured. Default to `true`.

Example
~~~~~~~

::

  [app_lib]
  recipe = tipfyrecipes.gae:eggs
  lib-directory = app/distlib
  use-zipimport = false

  # Define the libraries.
  eggs =
	  python-dateutil

  # Don't copy files that match these glob patterns.
  ignore-globs =
	  *.c
	  *.pyc
	  *.pyo
	  */test
	  */tests
	  */testsuite
	  */django
	  */sqlalchemy

  # Don't install these packages or modules.
  ignore-packages =
	  distribute
	  setuptools
	  easy_install
	  site
	  pkg_resources
"""
import datetime
import logging
import os
import shutil
import tempfile
import uuid

from z3c.recipe.scripts.scripts import Scripts
from tipfyrecipes import copytree, ignore_patterns, include_patterns, rmfiles, zipdir

BASE = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(
	os.path.abspath(os.path.realpath(__file__))))))

LIB_README = """Warning!
========

This directory is removed every time the buildout tool runs, so don't place
or edit things here because any changes will be lost!

Use a different directory for extra libraries instead of this one."""

class InstallAppLib(Scripts):
	def __init__(self, buildout, name, opts):
		# Set a logger with the section name.
		self.logger = logging.getLogger(name)

		# Unzip eggs by default or we can't use some.
		opts.setdefault('unzip', 'true')

		self.eggs_dir = buildout['buildout']['eggs-directory']
		self.parts_dir = buildout['buildout']['parts-directory']
		self.temp_dir = os.path.join(self.parts_dir, 'temp')

		lib_dir = opts.get('lib-directory', 'distlib')
		self.lib_path = os.path.abspath(lib_dir)

		self.use_zip = opts.get('use-zipimport', 'false') == 'true'
		if self.use_zip:
			self.lib_path += '.zip'

		# Set list of globs and packages to be ignored.
		self.ignore_globs = [i for i in opts.get('ignore-globs', '') \
			.splitlines() if i.strip()]
		self.ignore_packages = [i for i in opts.get('ignore-packages', '') \
			.splitlines() if i.strip()]

		self.delete_safe = opts.get('delete-safe', 'true') != 'false'
		opts.setdefault('eggs', '')
		super(InstallAppLib, self).__init__(buildout, name, opts)

	def install(self):
		# Get all installed packages.
		reqs, ws = self.working_set()
		paths = self.get_package_paths(ws)

		# For now we only support installing them in the app dir.
		# In the future we may support installing libraries in the parts dir.
		self.install_in_app_dir(paths)

		return super(InstallAppLib, self).install()

	update = install

	def install_in_app_dir(self, paths):
		# Delete old libs.
		self.delete_libs()

		if self.use_zip:
			# Create temporary directory for the zip files.
			tmp_dir = os.path.join(tempfile.gettempdir(), uuid.uuid4().hex)
		else:
			tmp_dir = self.lib_path

		if not os.path.exists(tmp_dir):
			os.mkdir(tmp_dir)

		# Copy all files.
		for name, src in paths:
			if name in self.ignore_packages:
				# This package or module must be ignored.
				continue

			dst = os.path.join(tmp_dir, name)
			if not os.path.isdir(src):
				# Try single files listed as modules.
				src += '.py'
				dst += '.py'
				if not os.path.isfile(src) or os.path.isfile(dst):
					continue

			self.logger.info('Copying %r...' % src)

			copytree(src, dst, os.path.dirname(src) + os.sep,
				ignore=ignore_patterns(*self.ignore_globs),
				logger=self.logger)

		# Save README.
		f = open(os.path.join(tmp_dir, 'README.txt'), 'w')
		f.write(LIB_README)
		f.close()

		if self.use_zip:
			# Zip file and remove temporary dir.
			zipdir(tmp_dir, self.lib_path)
			if os.path.isdir(tmp_dir):
				shutil.rmtree(tmp_dir)

	def get_package_paths(self, ws):
		"""Returns the list of package paths to be copied."""
		pkgs = []
		for path in ws.entries:
			lib_paths = self.get_lib_paths(path)
			if not lib_paths:
				self.logger.info('Library not installed: missing egg info for '
					'%r.' % path)
				continue

			for lib_path in lib_paths:
				pkgs.append((lib_path, os.path.join(path, lib_path)))

		return pkgs

	def get_top_level_libs(self, egg_path):
		top_path = os.path.join(egg_path, 'top_level.txt')
		if not os.path.isfile(top_path):
			return None

		f = open(top_path, 'r')
		libs = f.read().strip()
		f.close()

		# One lib per line.
		return [l.strip() for l in libs.splitlines() if l.strip()]

	def get_lib_paths(self, path):
		"""Returns the 'EGG-INFO' or '.egg-info' directory."""
		egg_path = os.path.join(path, 'EGG-INFO')
		if os.path.isdir(egg_path):
			# Unzipped egg metadata.
			return self.get_top_level_libs(egg_path)

		if os.path.isfile(path):
			# Zipped egg? Should we try to unpack it?
			# unpack_archive(path, self.eggs_dir)
			return None

		# Last try: develop eggs.
		elif os.path.isdir(path):
			files = os.listdir(path)
			for filename in files:
				if filename.endswith('.egg-info'):
					egg_path = os.path.join(path, filename)
					return self.get_top_level_libs(egg_path)

	def delete_libs(self):
		"""If the `delete-safe` option is set to true, move the old libraries
		directory to a temporary directory inside the parts dir instead of
		deleting it.
		"""
		if not os.path.exists(self.lib_path):
			# Nothing to delete, so it is safe.
			return

		if self.delete_safe is True:
			# Move directory or zip to temporary backup directory.
			if not os.path.exists(self.temp_dir):
				os.makedirs(self.temp_dir)

			date = datetime.datetime.now().strftime('_%Y_%m_%d_%H_%M_%S')
			filename = os.path.basename(self.lib_path.rstrip(os.sep))
			if self.use_zip:
				filename = filename[:-4] + date + '.zip'
			else:
				filename += date

			dst = os.path.join(self.temp_dir, filename)
			shutil.move(self.lib_path, dst)
			self.logger.info('Saved libraries backup in %r.' % dst)
		else:
			# Simply delete the directory or zip.
			if self.use_zip:
				os.remove(self.lib_path)
				self.logger.info('Removed lib-zip %r.' % self.lib_path)
			else:
				# Delete the directory.
				shutil.rmtree(self.lib_path)
				self.logger.info('Removed lib-directory %r.' % self.lib_path)



# Carlo Pires <carlopires@gmail.com>
# Seg 14 Mar 2011 21:46:55 BRT 
import logging, os, zc.buildout

class CreateSymbolicLinks:
	"""
	Buildout recipe for create dir and link modules. Use as:

	[buildout]
	parts = 
		django
		tipfy

	[django]
	recipe = recipes:ln
	target = /home/carlo/django/trunk/django
	directory = app/lib/dist/django

	[tipfy]
	recipe = recipes:ln
	directory = app/lib/dist
	target = /home/carlo/tipfy/tipfy
	target_names = 
		tipfy
		tipfyext


	"""
	def __init__(self, buildout, name, options):
		self.name, self.options = name, options
		self.logger = logging.getLogger(self.name)

		if 'directory' not in self.options:
			raise zc.buildout.UserError('Link directory must be provided')
		self.directory = self.options['directory']

		if 'target' not in self.options:
			raise zc.buildout.UserError('Target directory must be provided')
		self.target = self.options['target']

		if 'target_names' in self.options:
			self.targets = [s.strip() for s in self.options['target_names'].split('\n') if len(s) > 0]
		else:
			self.targets = None

	def create_symbolic_link(self, target, directory):
		if not os.path.exists(os.path.dirname(directory)):
			os.makedirs(os.path.dirname(directory))
		if not os.path.exists(directory):
			target = os.path.realpath(target)
			self.logger.info('Creating symbolic link %s -> %s', directory, target)
			os.symlink(target, directory)

	def install(self):
		if self.targets:
			for link_name in self.targets:
				target = os.path.join(self.target, link_name)
				directory = os.path.join(self.directory, link_name)
				self.create_symbolic_link(target, directory)
		else:
			self.create_symbolic_link(self.target, self.directory)

		return self.targets or self.target

	def update(self):
		pass

def uninstall(name, options):
	logger = logging.getLogger(name)

	if 'directory' not in options:
		raise zc.buildout.UserError('Link directory must be provided')
	directory = options['directory']

	if 'target' not in options:
		raise zc.buildout.UserError('Target directory must be provided')
	target = options['target']

	if 'target_names' in options:
		targets = [s.strip() for s in options['target_names'].split('\n') if len(s) > 0]
	else:
		targets = None

	if targets:
		for link_name in targets:
			path = os.path.join(directory, link_name)
			try:
				os.readlink(path)
				os.unlink(path)
			except:
				pass
	else:
		try:
			os.readlink(directory)
			os.unlink(directory)
		except:
			pass




# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Qua 16 Mar 2011 08:59:15 BRT  
# -*- coding: utf-8 -*-
"""
General utilities shared by all recipes.
"""
import fnmatch
import os
import shutil
import zipfile

def get_relative_path(path, base_path):
	path = os.path.normcase(path)
	base_path = os.path.normcase(base_path)

	original = path
	r = []
	while 1:
		dirname, basename = os.path.split(path)
		r.append(basename)
		if dirname == base_path:
			break

		if dirname == path:
			return '%r' % original

		path = dirname

	r.reverse()
	return 'join(base, %r)' % os.path.join(*r)

def copytree(src, dst, dirname, symlinks=False, ignore=None, logger=None):
	"""Recursively copy a directory tree using copy2().

	The destination directory must not already exist.
	If exception(s) occur, an Error is raised with a list of reasons.

	If the optional symlinks flag is true, symbolic links in the
	source tree result in symbolic links in the destination tree; if
	it is false, the contents of the files pointed to by symbolic
	links are copied.

	The optional ignore argument is a callable. If given, it
	is called with the `src` parameter, which is the directory
	being visited by copytree(), and `names` which is the list of
	`src` contents, as returned by os.listdir():

		callable(src, names) -> ignored_names

	Since copytree() is called recursively, the callable will be
	called once for each directory that is copied. It returns a
	list of names relative to the `src` directory that should
	not be copied.

	XXX Consider this example code rather than the ultimate tool.

	Adapted from Python 2.6 source.
	"""
	ignored_names = set()

	if os.path.isfile(src):
		if ignore is not None:
			ignored_names = ignore(os.path.dirname(src)[len(dirname):],
				[os.path.basename(src)])

		if src[len(dirname):] not in ignored_names:
			shutil.copyfile(src, dst)

		return

	names = os.listdir(src)
	if ignore is not None:
		ignored_names = ignore(src[len(dirname):], names)

	if os.path.isdir(dst):
		if logger:
			logger.info('%r already exists and will not be created.' % dst)
	else:
		os.makedirs(dst)

	errors = []
	for name in names:
		srcname = os.path.join(src, name)
		dstname = os.path.join(dst, name)
		if srcname[len(dirname):] in ignored_names:
			continue
		try:
			if symlinks and os.path.islink(srcname):
				linkto = os.readlink(srcname)
				os.symlink(linkto, dstname)
			elif os.path.isdir(srcname):
				copytree(srcname, dstname, dirname, symlinks, ignore,
					logger=logger)
			else:
				if os.path.isfile(dstname):
					if logger:
						logger.info('%r already exists and will not be '
							'created.' % dstname)
				else:
					shutil.copy2(srcname, dstname)
			# XXX What about devices, sockets etc.?
		except (IOError, os.error), why:
			errors.append((srcname, dstname, str(why)))
		# catch the Error from the recursive copytree so that we can
		# continue with other files
		except shutil.Error, err:
			errors.extend(err.args[0])
	try:
		shutil.copystat(src, dst)
	except OSError, why:
		if WindowsError is not None and isinstance(why, WindowsError):
			# Copying file access times may fail on Windows
			pass
		else:
			errors.extend((src, dst, str(why)))
	if errors:
		raise shutil.Error, errors

def ignore_patterns(*patterns):
	"""Function that can be used as copytree() ignore parameter.

	Patterns is a sequence of glob-style patterns
	that are used to exclude files

	Adapted from Python 2.6 source.
	"""
	def _ignore_patterns(path, names):
		names = [os.path.join(path, name) for name in names]
		ignored_names = []
		for pattern in patterns:
			ignored_names.extend(fnmatch.filter(names, pattern))

		return set(ignored_names)
	return _ignore_patterns

include_patterns = ignore_patterns

def rmfiles(src, only=None):
	names = os.listdir(src)

	if only is not None:
		only_names = only(src, names)
	else:
		only_names = set()

	for name in names:
		srcname = os.path.join(src, name)
		if srcname in only_names:
			if os.path.isfile(srcname):
				os.remove(srcname)
			elif os.path.isdir(srcname):
				shutil.rmtree(srcname)
		elif os.path.isdir(srcname):
			rmfiles(srcname, only=only)

def zipdir(dirname, filename):
	assert os.path.isdir(dirname)
	z = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
	try:
		for root, dirs, files in os.walk(dirname):
			# NOTE: ignore empty directories
			for f in files:
				absf = os.path.join(root, f)
				zf = absf[len(dirname)+len(os.sep):]
				z.write(absf, zf)

		z.close()
	finally:
		if z:
			z.close()



# -o- coding: utf-8 -o-
# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Ter 15 Mar 2011 18:01:52 BRT 

import logging
import os.path
import shutil
import tempfile
import urlparse

import zc.buildout
from zc.buildout.download import ChecksumError, Download

import setuptools.archive_util

from tipfyrecipes.utils import get_bool_option, get_checksum

class Recipe(object):
    """
    Recipe for downloading packages from the net and
    extracting them on the filesystem.
    """
    def __init__(self, buildout, name, options):
        self.options = options
        self.buildout = buildout
        self.name = name

        # Set a logger with the section name.
        self.logger = logging.getLogger(name)

        self.download_cache = buildout['buildout'].setdefault('download-cache',
            os.path.join(buildout['buildout']['directory'], 'downloads'))

        # All options
        self.option_url = options.get('url')
        self.option_md5sum = options.get('md5sum')
        self.option_sha1sum = options.get('sha1sum')
        self.option_destination = options.setdefault('destination',
            os.path.join(buildout['buildout']['parts-directory'], self.name))
        self.option_clear_destination = get_bool_option(
            options.setdefault('clear-destination', 'false'))
        self.option_strip_top_level_dir = get_bool_option(
            options.setdefault('strip-top-level-dir', 'false'))
        self.option_download_only = get_bool_option(
            options.setdefault('download-only', 'false'))
        self.option_hash_name = get_bool_option(
            options.setdefault('hash-name', 'false'))
        self.option_filename = options.get('filename', '').strip()

    def install(self):
        if not os.path.exists(self.download_cache):
            os.makedirs(self.download_cache)

        cached_path, is_temp = self.download()

        parts = []

        try:
            # Create destination directory
            if not os.path.isdir(self.option_destination):
                os.makedirs(self.option_destination)
                parts.append(self.option_destination)

            if self.option_download_only:
                if self.option_filename:
                    # Use an explicit filename from the section configuration
                    filename = self.option_filename
                else:
                    # Use the original filename of the downloaded file
                    # regardless whether download filename hashing is enabled.
                    # See http://github.com/hexagonit/hexagonit.recipe.download/issues#issue/2
                    filename = os.path.basename(urlparse.urlparse(self.option_url)[2])

                # Copy the file to destination without extraction
                target_path = os.path.join(self.option_destination, filename)
                shutil.copy(cached_path, target_path)
                if not self.option_destination in parts:
                    parts.append(target_path)
            else:
                # Extract the package
                extract_dir = tempfile.mkdtemp("buildout-" + self.name)
                try:
                    setuptools.archive_util.unpack_archive(cached_path,
                        extract_dir)
                except setuptools.archive_util.UnrecognizedFormat:
                    self.logger.error('Unable to extract the package %s. '
                        'Unknown format.', cached_path)
                    raise zc.buildout.UserError('Package extraction error')

                base = self.calculate_base(extract_dir)

                if not os.path.isdir(self.option_destination):
                    os.makedirs(self.option_destination)

                self.logger.info('Extracting package to %s' %
                    self.option_destination)

                for filename in os.listdir(base):
                    dest = os.path.join(self.option_destination, filename)
                    if os.path.exists(dest):
                        if self.option_clear_destination:
                            shutil.rmtree(dest)
                            self.logger.info('Removed: %r.' % dest)
                            parts.append(dest)
                        else:
                            self.logger.error(
                                'Target %s already exists. Either remove it '
                                'or set ``clear-destination = true`` in your '
                                'buildout.cfg to remove existing files and '
                                'directories before moving downloaded files.',
                                dest)
                            raise zc.buildout.UserError('File or directory '
                                'already exists.')
                    else:
                        # Only add the file/directory to the list of installed
                        # parts if it does not already exist. This way it does
                        # not get accidentally removed when uninstalling.
                        parts.append(dest)

                    if not os.path.exists(dest):
                        shutil.move(os.path.join(base, filename), dest)

                shutil.rmtree(extract_dir)

        finally:
            if is_temp:
                os.unlink(cached_path)

        return parts

    def update(self):
        pass

    def calculate_base(self, extract_dir):
        """
        recipe authors inheriting from this recipe can override this method
        to set a different base directory.
        """
        # Move the contents of the package in to the correct destination
        top_level_contents = os.listdir(extract_dir)
        if self.option_strip_top_level_dir:
            if len(top_level_contents) != 1:
                self.logger.error('Unable to strip top level directory '
                    'because there are more than one element in the root '
                    'of the package.')
                raise zc.buildout.UserError('Invalid package contents')

            base = os.path.join(extract_dir, top_level_contents[0])
        else:
            base = extract_dir

        return base

    def download(self):
        d = Download(self.buildout['buildout'],
            hash_name=self.option_hash_name)
        cached_path, is_temp = d(self.option_url, md5sum=self.option_md5sum)

        if self.option_sha1sum and \
            self.option_sha1sum != get_checksum(cached_path):
            raise ChecksumError('SHA1 checksum mismatch for cached download '
                'from %r at %r' % (self.option_url, cached_path))

        return cached_path, is_temp



# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# forked from: http://pypi.python.org/pypi/hgrecipe/0.9
# Qua 16 Mar 2011 09:09:01 BRT

import logging, os, zc.buildout, subprocess

class Git(object):
	'''Simple recipe for fetch code form remote repository, using system git'''
	def __init__(self, buildout, name, options):
		self.options, self.buildout = options, buildout
		self.logger = logging.getLogger(name)

		if 'repository' not in self.options:
			raise zc.buildout.UserError('Repository url must be provided')
		self.url = options['repository']

		if 'directory' not in self.options:
			raise zc.buildout.UserError('Directory url must be provided')
		self.directory = options['directory']
		
		# ref option overrides rev
		if 'rev' in options:
			self.ref = options.get('rev', 'origin/master')
		if 'ref' in options:
			self.ref = options.get('ref', 'origin/master')

	def git(self, operation, args):
		command = ' '.join(['git', operation, '-q'] + args)
		self.logger.info(command)
		status = subprocess.call(command, shell=True)
		if status != 0:
			raise zc.buildout.UserError('Error while executing %s' % ' '.join(command))

	def install(self):
		'''Clone repository and checkout to version'''
		os.chdir(self.buildout['buildout']['directory'])
		try:
			self.git('clone', [self.url, self.directory])
			if 'rev' in self.options:
				os.chdir(self.options['directory'])
				self.git('checkout', [self.ref, ])
		except zc.buildout.UserError:
			# should manually delete files because buildout thinks that no files created
			from shutil import rmtree
			rmtree(self.options['directory'])
			raise

		os.chdir(self.buildout['buildout']['directory'])
		return self.options['directory']

	def update(self):
		'''Update repository rather than download it again'''
		os.chdir(self.options['directory'])
		self.git('fetch', ['origin', ])

		if 'rev' in self.options:
			self.git('checkout', [self.ref, ])

		os.chdir(self.buildout['buildout']['directory'])

		return self.options['directory']

def uninstall(name, options):
	"""
	Remove the old repository if ``overwrite`` is marked as true.
	Otherwise, leave it alone.
	"""
	if options.get('overwrite') == 'true':
		import shutil
		shutil.rmtree(options.get('directory'), ignore_errors=True)



# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Qua 16 Mar 2011 09:23:36 BRT 

"""
Installs a python executable and several SDK scripts in the buildout
directory: appcfg, bulkload_client, bulkloader, dev_appserver and
remote_api_shell.

It also allows to set default values to start the dev_appserver.

This recipe extends `zc.recipe.egg.Scripts <http://pypi.python.org/pypi/zc.recipe.egg>`_,
so all the options from that recipe are also valid.

Options
~~~~~~~

:sdk-directory: Path to the App Engine SDK directory. It can be an
	absolute path or a reference to the `appfy.recipe.gae:sdk` destination
	option. Default is `${buildout:parts-directory}/google_appengine`.
:appcfg-script: Name of the appcfg script to be installed in the bin
	directory.. Default is `appcfg`.
:bulkload_client-script: Name of the bulkloader script to be installed in
	the bin directory. Default is `bulkload_client`.
:bulkloader-script: Name of the bulkloader script to be installed in
	the bin directory. Default is `bulkloader`.
:dev_appserver-script: Name of the dev_appserver script to be installed in
	the bin directory. Default is `dev_appserver`.
:remote_api_shell-script: Name of the remote_api_shell script to be
	installed in the bin directory. Default is `remote_api_shell`.
:config-file: Configuration file with the default values to use in
	scripts. Default is `gaetools.cfg`.
:extra-paths: Extra paths to include in sys.path for generated scripts.

Example
~~~~~~~

::

	[tools]
	recipe = tipfyrecipes:tools
	sdk-directory = ${gae_sdk:destination}/google_appengine

	# Add these paths to sys.path in the generated scripts.
	extra-paths =
		src/lib
		src/distlib

To set default values to start the dev_appserver, create a section
`dev_appserver` in the defined configuration file (`gaetools.cfg` by
default). For example::

	[dev_appserver]
	# Set default values to start the dev_appserver. All options from the
	# command line are allowed. They are inserted at the beginning of the
	# arguments. Values are used as they are; don't use variables here.
	recipe = tipfyrecipes.gae:tools
	defaults =
		--datastore_path=var/data.store
		--history_path=var/history.store
		--blobstore_path=var/blob.store
		src

Each option should be set in a separate line, as displayed above. Options
provided when calling dev_appserver will override the default values.
"""
import logging
import os

import zc.recipe.egg

from tipfyrecipes import get_relative_path

BASE = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(
	os.path.abspath(os.path.realpath(__file__))))))

class InstallGAETools(zc.recipe.egg.Scripts):
	def __init__(self, buildout, name, opts):
		self.parts_dir = buildout['buildout']['parts-directory']
		self.buildout_dir = buildout['buildout']['directory']

		# Set default values.
		join = os.path.join
		defaults = {
			'sdk-directory': join(self.parts_dir, 'google_appengine'),
			'config-file':   join(self.buildout_dir, 'gaetools.cfg'),
			'interpreter':   'python',
			'extra-paths':   '',
			'eggs':		  '',
		}
		defaults.update(opts)
		opts = defaults

		# Set normalized paths.
		self.config_file = os.path.abspath(opts['config-file'])
		self.sdk_dir = os.path.abspath(opts['sdk-directory'])

		# Set the scripts to be generated.
		scripts = ['appcfg',
				   'bulkload_client',
				   'bulkloader',
				   'dev_appserver',
				   'remote_api_shell']

		self.scripts = [(s, opts.get(s + '-script', s)) for s in scripts]

		# Add the SDK and this recipe package to the path.
		opts['extra-paths'] += '\n%s\n%s' % (BASE, self.sdk_dir)

		# Set a flag to use relative paths.
		self.use_rel_paths = opts.get('relative-paths',
			buildout['buildout'].get('relative-paths', 'false')) == 'true'

		super(InstallGAETools, self).__init__(buildout, name, opts)

	def install(self):
		"""Creates the scripts."""
		entry_points =['%s=tipfyrecipes.gae.scripts:%s' % (scriptname,
			function) for function, scriptname in self.scripts]

		if self.use_rel_paths is not True:
			# base won't be set if we are using absolute paths.
			initialization = ['base = %r' % self.buildout_dir]
		else:
			initialization = []

		initialization.append('gae = %s' % self.get_path(self.sdk_dir))
		initialization.append('cfg = %s' % self.get_path(self.config_file))

		self.options.update({
			'entry-points':   ' '.join(entry_points),
			'initialization': '\n'.join(initialization),
			'arguments':	  'base, gae, cfg',
		})

		return super(InstallGAETools, self).install()

	def get_path(self, path):
		if self.use_rel_paths is True:
			return get_relative_path(path, self.buildout['buildout']['directory'])
		else:
			return '%r' % os.path.abspath(path)

	update = install



# -*- coding: utf-8 -*-
# Carlo Pires <carlopires@gmail.com>
# from http://pypi.python.org/pypi/appfy.recipe.gae/0.9.3
# Ter 15 Mar 2011 17:58:14 BRT 

import os

from tipfyrecipes.download import Recipe as DownloadRecipe

class InstallGAE(DownloadRecipe):
	"""
	Downloads and installs the App Engine SDK in the buildout directory.

	Options
	~~~~~~~

	:url: URL to the App Engine SDK file.
	:destination: Destination of the extracted SDK. Default is the parts directory.
	:clear-destination: If `true`, deletes the destination dir before
		extracting the download. Default is `true`.

	Example
	~~~~~~~

	::

	  [gae_sdk]
	  recipe = appfy.recipe.gae:sdk
	  url = http://googleappengine.googlecode.com/files/google_appengine_1.4.2.zip
	  destination = ${buildout:parts-directory}
	  hash-name = false
	  clear-destination = true
	"""
	def __init__(self, buildout, name, options):
		parts_dir = os.path.abspath(buildout['buildout']['parts-directory'])
		options.setdefault('destination', parts_dir)
		options.setdefault('clear-destination', 'true')
		super(InstallGAE, self).__init__(buildout, name, options)



from setuptools import setup, find_packages

setup(
    name = 'tipfyrecipes',
    version = '0.2',
    license = 'BSD',
    keywords='buildout recipe google app engine appengine gae zc.buildout tipfy',
    url='http://code.google.com/p/tipfyrecipes/',
    description = 'Buildout recipes for Tipfy web framework',
    author = 'Carlo Pires',
    author_email = 'carlopires@gmail.com',
    zip_safe = False,
    platforms = 'any',
    packages = find_packages(),
    install_requires=[
        'setuptools',
        'zc.buildout >= 1.5.2',
        'z3c.recipe.scripts >= 1.0.1',
        'Mercurial',
        'Mock==0.6.0',
    ],
	entry_points = {
		'zc.buildout': [
			'sdk = tipfyrecipes.gae.sdk:InstallGAE',
			'tools = tipfyrecipes.gae.tools:InstallGAETools',
			'applib = tipfyrecipes.gae.applib:InstallAppLib',
			'git = tipfyrecipes.git:Git',
			'hg = tipfyrecipes.hg:Mercurial',
			'svn = tipfyrecipes.svn:Subversion',
			'ln = tipfyrecipes.ln:CreateSymbolicLinks',
		],
		'zc.buildout.uninstall': [
			'git = tipfyrecipes.git:uninstall',
			'hg = tipfyrecipes.hg:uninstall',
			'svn = tipfyrecipes.svn:uninstall',
			'ln = tipfyrecipes.ln:uninstall',
		],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Buildout',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# -*- coding: utf-8 -*-
import ConfigParser
import os
import runpy
import sys

def parse_argv(argv):
    """A very simple argv parser. Returns a list of (opts, args) for options
    and arguments in the passed argv. It won't try to validate anything.
    """
    opts = []
    args = []
    for arg in argv:
        if arg.startswith('-'):
            parts = arg.split('=', 1)
            if len(parts) == 1:
                val = (parts[0], None)
            else:
                val = (parts[0], parts[1])

            opts.append(val)
        else:
            args.append(arg)

    return opts, args

def merge_opts(sys_opts, def_opts):
    """Merges options from sys.argv to default options: the first overrides
    the later.
    """
    res = []
    defined_opts = dict(def_opts)
    defined_keys = defined_opts.keys()

    for opt, optarg in sys_opts:
        if opt in defined_keys:
            defined_keys.remove(opt)

        if optarg is None:
            res.append(opt)
        else:
            res.append('%s=%s' % (opt, optarg))

    for opt in defined_keys:
        optarg = defined_opts[opt]

        if optarg is None:
            res.append(opt)
        else:
            res.append('%s=%s' % (opt, optarg))

    return res

def get_dev_appserver_argv(defaults):
    def_opts, def_args = parse_argv(defaults)
    sys_opts, sys_args = parse_argv(sys.argv[1:])

    app = None
    if sys_args:
        app = sys_args[0]
    elif def_args:
        app = def_args[0]

    res = [sys.argv[0]] + merge_opts(sys_opts, def_opts)
    if app:
        res.append(app)

    return res

def get_config(filename, section=None, key=None):
    try:
        config = ConfigParser.RawConfigParser()
        config.read(filename)
        if section is None:
            return config

        values = config.items(section)
        if key is None:
            return values

        return dict(values).get(key, None)
    except ConfigParser.NoSectionError:
        return None

def get_dev_appserver_config(config_file):
    config = get_config(config_file, 'dev_appserver', 'defaults')
    if config:
        # Only accept multi-line configuration.
        config = [o.strip() for o in config.splitlines() if o.strip()]

    if not config:
        return None

    return config

def appcfg(base, gae_path, config_file):
    runpy.run_module('appcfg', run_name='__main__', alter_sys=True)

def bulkload_client(base, gae_path, config_file):
    runpy.run_module('bulkload_client', run_name='__main__', alter_sys=True)

def bulkloader(base, gae_path, config_file):
    runpy.run_module('bulkloader', run_name='__main__', alter_sys=True)

def dev_appserver(base, gae_path, config_file):
    config = get_dev_appserver_config(config_file)
    if config:
        sys.argv = get_dev_appserver_argv(config)

    runpy.run_module('dev_appserver', run_name='__main__', alter_sys=True)

def remote_api_shell(base, gae_path, config_file):
    runpy.run_module('remote_api_shell', run_name='__main__', alter_sys=True)
